/*
 * lib_timer32.c
 *
 *  Created on:
 *      Author:
 */

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include "lib_timer.h"

extern uint16_t pwm_ls_power_on;
extern unsigned char hw_version;


// ***************************************************************************
//initialize timer16_0 --> create a pwm output for LS_LED_ON
// ***************************************************************************
void CT16B0_Init(void)
{
	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 7); // enable general clock for 16-bit timer 0

	LPC_TMR16B0->TCR = 1<<1; // reset timer

	// select prescale value
	LPC_TMR16B0->PR = 0;

	LPC_TMR16B0->CTCR = 0; // select timer mode

	LPC_TMR16B0->PWMC = 0x01; // stop every internal pwm
	LPC_TMR16B0->MCR = TIM_MR3R;  // timer16_0 match 3 irq (used as overflow irq !!!

	LPC_TMR16B0->EMR = 0; // no external match for the pwm

	LPC_TMR16B0->MR0 = 48; // initialized set output to zero
	LPC_TMR16B0->MR3 = 96;

	// enable timer
	LPC_TMR16B0->TCR = 1;
}


// ***************************************************************************
//initialize timer16_1 --> 1mys counter (tAbsolute)
// ***************************************************************************
void CT16B1_Init(void)
{
	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 8); // enable general clock for 16-bit timer 1

	LPC_TMR16B1->TCR = 1<<1; // reset timer

	// select prescale value
	LPC_TMR16B1->PR = (SystemCoreClock / 1000000) -1; // prescaler set to 1mys Timer

	LPC_TMR16B1->CTCR = 0; // select timer mode

	LPC_TMR16B1->MCR = TIM_MR3I | TIM_MR3R;  // timer16_0 match 3 irq (used as overflow irq !!!

	LPC_TMR16B1->MR3 = 99; // 0-99 means 100 mys IRQ -->

	NVIC_EnableIRQ(TIMER_16_1_IRQn); // global enable the TIMER16_0 IRQ

	// enable timer
	LPC_TMR16B1->TCR = 1;
}


void CT32B0_Init(void)
{
	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 9);

	LPC_IOCON->R_PIO0_11 &= ~0x07;
	LPC_IOCON->R_PIO0_11 |= 0x03; /* Timer0_32 MAT3 */

	// disable and reset timer/counter
	LPC_TMR32B0->TCR = (1 << 1);
	// select prescale value
	LPC_TMR32B0->PR = SystemCoreClock / 1000000;
	// select timer mode
	LPC_TMR32B0->CTCR = TIM_TIMER_MODE;
	// release timer reset
	LPC_TMR32B0->TCR &= ~(1 << 1);
	// Capture 0 on rising and falling edge, interrupt enable.
	LPC_TMR32B0->CCR = 0; //(0x1 << 0) | (0x1 << 1) | (0x1 << 2);
	// select timer match behavior
	LPC_TMR32B0->MCR = TIM_MR3I | TIM_MR3R; // | TIM_MR2I; // Reset on MR3, Interrupt on MR2
	LPC_TMR32B0->EMR = 0; // no pin actions on match events
	// select first match value
	LPC_TMR32B0->MR3 = 1000;
	// Enable the TIMER1 Interrupt
	NVIC_EnableIRQ(TIMER_32_0_IRQn);
	// enable timer
	LPC_TMR32B0->TCR = 1;
}


// initialize CT32B1 (P1.0-CT32B1_CAP0)
void CT32B1_Init(void)
{
	// ATTENSION ** ATTENSION ** ATTENSION ** ATTENSION
	// Some of the I/O pins need to be carefully planned because
	// JTAG SWCLK (Debugging) and TIMER32 1 MATCH 2 functions are muxed.
	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 10);

	LPC_TMR32B1->TCR = (1 << 1); 					// disable and reset timer/counter
	LPC_TMR32B1->PR = SystemCoreClock / 1000000;	// select prescale value for 1MHz
	LPC_TMR32B1->CTCR = TIM_TIMER_MODE;				// select timer mode

	LPC_TMR32B1->PWMC = 0x0A; // switch to internal PWM 0/1/3
	// select timer match behavior
	LPC_TMR32B1->MCR = 0x49;		// an IRQ is generatet if match 0 and match 2

	LPC_TMR32B1->EMR = 0x8A0; // ports clear at match value
	/* select first match value */
	LPC_TMR32B1->MR0 = 0;
	LPC_TMR32B1->MR1 = 0;
	LPC_TMR32B1->MR2 = 4000; // 1040 mys (approximately 1kHz (exactly 961,5 Hz) max. PWM length
	LPC_TMR32B1->MR3 = 0;

	/* Enable the TIMER1 Interrupt */
	NVIC_EnableIRQ(TIMER_32_1_IRQn); //maybe we don´t need the irq

	LPC_TMR32B1->TCR = 1; // enable timer
}




/*void DelayExecution(uint32_t delay)
{
	uint32_t usTickCount = LPC_TMR32B1->TC;
	while ((LPC_TMR32B1->TC - usTickCount) < delay)
	{
		//wait
	}
}
*/
