/****************************************************************************
 *   $Id:: main.c 5034 2010-09-24 23:48:37Z nxp12832                        $
 *   Project: NXP LPC11xx CAN example
 *
 *   Description:
 *     This file is part of the CAN and CANopen on-chip driver examples.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#include "driver_config.h"
#include "target_config.h"

#include "rom_drivers.h"
#include "lib_timer.h"
#include "Peripheral_HW.h"
#include "i2c.h"
#include"tsl2561.h"
#include "lm75b.h"


#ifndef NULL
#define NULL    ((void *)0)
#endif

unsigned long tAbsolute;
unsigned char tAbsPreCnt;
uint32_t Temperatur;

unsigned long mkCANcycle;



volatile uint8_t ID;
#define TX_CAN_OBJ	0x100



int32_t temp = 0;
uint16_t broadband, ir;
uint32_t lux;



ROM **rom = (ROM **)0x1fff1ff8;

CAN_MSG_OBJ msg_obj;

/* Initialize CAN Controller */
#if 0
uint32_t ClkInitTable[2] = {
  0x00000000UL, // CANCLKDIV
  0x00001C57UL  // CAN_BTR
};
#elif 0
uint32_t ClkInitTable[2] = {
  0x00000000UL, // CANCLKDIV
  0x00001BE5UL  // CAN_BTR 615kBaud
};
#else
uint32_t ClkInitTable[2] = {
  0x00000005UL, // CANCLKDIV
  0x00002301UL  // CAN_BTR 500kBaud
};
#endif

/* Callback function prototypes */
void CAN_rx(uint8_t msg_obj_num);
void CAN_tx(uint8_t msg_obj_num);
void CAN_error(uint32_t error_info);

/* Publish CAN Callback Functions */
CAN_CALLBACKS callbacks = {
   CAN_rx,
   CAN_tx,
   CAN_error,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
};

/*	CAN receive callback */
/*	Function is executed by the Callback handler after
	a CAN message has been received */
void CAN_rx(uint8_t msg_obj_num){
  
  /* Determine which CAN message has been received */
  msg_obj.msgobj = msg_obj_num;

  /* Now load up the msg_obj structure with the CAN message */
  (*rom)->pCAND->can_receive(&msg_obj);

  if (msg_obj_num == 1)
  {
    /* Simply transmit CAN frame (echo) with with ID +0x100 via buffer 2 */
    msg_obj.msgobj = 2;
    msg_obj.dlc = 2;
    msg_obj.mode_id += 0x100;
    (*rom)->pCAND->can_transmit(&msg_obj);
  }

  return;
}

/*	CAN transmit callback */
/*	Function is executed by the Callback handler after
	a CAN message has been transmitted */
void CAN_tx(uint8_t msg_obj_num){
  return;
}

/*	CAN error callback */
/*	Function is executed by the Callback handler after
	an error has occured on the CAN bus */
void CAN_error(uint32_t error_info){
  return;
}

/*	CAN interrupt handler */
/*	The CAN interrupt handler must be provided by the user application.
	It's function is to call the isr() API located in the ROM */
void CAN_IRQHandler (void){
  (*rom)->pCAND->isr();
}

void init(void){

	//Initialise the LM75B
	lm75bInit();

	//Initialise Timer
	CT16B1_Init();


	//Initialise Ports
	SetPortPinDir(IO_PORT_1,0,IN);
	SetPortPinDir(IO_PORT_1,1,IN);
	SetPortPinDir(IO_PORT_1,2,IN);
	SetPortPinDir(IO_PORT_1,3,IN);
	SetPortPinDir(IO_PORT_1,4,IN);
	SetPortPinDir(IO_PORT_1,5,IN);

	SetPortPinDir(IO_PORT_3,0,OUT);


}

void TIMER16_1_IRQHandler(void)
{
	tAbsPreCnt++;
	if (tAbsPreCnt >= 10 )
	{
      tAbsPreCnt = 0;
      tAbsolute++; // 1ms Counter !!!
	}

LPC_TMR16B1->IR = LPC_TMR16B1->IR; 	// clear all pending interrupts
}

void SendCanData(uint8_t ID, uint8_t msg_obj_lengh) {
			msg_obj.msgobj  = 0;
			msg_obj.mode_id = ID;
			msg_obj.mask    = 0x0;
			msg_obj.dlc     = msg_obj_lengh;
			(*rom)->pCAND->can_transmit(&msg_obj);
}



void cycled200ms (void){
	if ( (tAbsolute - mkCANcycle) > 200 ){
		mkCANcycle = tAbsolute;
		SendCanData(TX_CAN_OBJ+ID,8);
		lm75bGetTemperature(&temp);
		Temperatur = temp * 125 /100;
	}
}


void CanBuffer (void){

	msg_obj.data[0] = TasterPort;
	msg_obj.data[1] = Temperatur;
}

//Funktion zu neustarten der CPU...
void reboot(void)
{
	NVIC_SystemReset();
}


int main(void) {

	init();
	//SetPortPin(IO_PORT_3,0,1);

	/* Output the Clk onto the CLKOUT Pin PIO0_1 to monitor the freq on a scope */
	LPC_IOCON->PIO0_1	= (1<<0);
	/* Select the MAIN clock as the clock out selection since it's driving the core */
	LPC_SYSCON->CLKOUTCLKSEL = 3;
	/* Set CLKOUTDIV to 6 */
	LPC_SYSCON->CLKOUTDIV = 10;		//	CLKOUT Divider = 10
	/* Enable CLKOUT */
	LPC_SYSCON->CLKOUTUEN = 0;
	LPC_SYSCON->CLKOUTUEN = 1;
	while (!(LPC_SYSCON->CLKOUTUEN & 0x01));

	/* Initialize the CAN controller */
	(*rom)->pCAND->init_can(&ClkInitTable[0], 1);

	/* Configure the CAN callback functions */
	(*rom)->pCAND->config_calb(&callbacks);

	/* Enable the CAN Interrupt */
	NVIC_EnableIRQ(CAN_IRQn);

	/* Configure message object 1 to receive all 11-bit messages 0x400-0x4FF */
	msg_obj.msgobj = 1;
	msg_obj.mode_id = 0x400;
	msg_obj.mask = 0x700;
	(*rom)->pCAND->config_rxmsgobj(&msg_obj);	
	int32_t temp = 0;


	while(1){

		cycled200ms();

		TasterPort = GetPortValue(IO_PORT_1);

		CanBuffer();

		//__WFI();	/* Go to Sleep */
	}
}





